/** Rill v0.0.471 https://www.npmjs.com/package/rill License: MIT*/
var byteLength, empty, isType;

isType = require("./isType");

byteLength = require("./byteLength");

empty = {
  204: true,
  205: true,
  304: true
};


/*
 * A default middleware that handles sending the response for the request.
#
 * @param {Object} req
 * @param {Object} res
 * @param {Function} next
 */

module.exports = function*(req, res, next) {
  var body, headers, method, originalRes, redirect, status;
  (yield next);
  method = req.method, redirect = req.redirect;
  originalRes = res.originalRes, headers = res.headers, status = res.status, body = res.body;
  if (originalRes.headersSent) {
    return;
  }
  if (redirect != null) {
    if (headers["Location"] == null) {
      headers["Location"] = redirect;
    }
  }
  if (Number(status) === 404 && (body != null)) {
    status = 200;
  }
  if (empty[status] || (body == null)) {
    if (headers["Content-Type"] == null) {
      headers["Content-Type"] = void 0;
    }
    body = void 0;
  } else if (isType.String(body)) {
    if (headers["Content-Type"] == null) {
      headers["Content-Type"] = "text/" + (/^\s*</.test(body) ? "html" : "plain");
    }
  } else if (isType.Object(body)) {
    if (headers["Content-Type"] == null) {
      headers["Content-Type"] = "application/json";
    }
    body = JSON.stringify(body);
  } else {
    if (headers["Content-Type"] == null) {
      headers["Content-Type"] = "application/octet-stream";
    }
    if (body.length) {
      if (headers["Content-Length"] == null) {
        headers["Content-Length"] = body.length;
      }
    }
    if (isType.Stream(body)) {
      return body.pipe(originalRes);
    }
  }
  if (headers["Content-Length"] == null) {
    headers["Content-Length"] = byteLength(body);
  }
  {
    originalRes.writeHead(status, headers);
    return originalRes.end("HEAD" !== method ? body : void 0);
  }
};
